///////////////////////////////////////////////////////////////////////////////
// Name:        XercesCDependent/swXMLChStr.h
// Author:      Reinhold Freder
// Created:     2004
// Copyright:   (c) 2005 Reinhold Freder
// Licence:     wxWindows licence
///////////////////////////////////////////////////////////////////////////////

#ifndef SWXMLCHSTR_H
#define SWXMLCHSTR_H

#ifdef __GNUG__
    #pragma interface "swXMLChStr.h"
#endif

#include "xercesc/util/PlatformUtils.hpp"

namespace sw {


/*! \class XMLChStr
    \brief Easy transcoding of char* data to XMLCh* data (Xerces).

    Strongly based on CreateDOMDocument project of standard Xerces examples.
    The main purpose for its existence is preventing memory leaks.
*/
class XMLChStr
{
public:
    /*! \fn XMLChStr (const char * const toTranscode)
        \brief Constructor.

        \param toTranscode char* data to convert
    */
    XMLChStr (const char * const toTranscode);


    /*! \fn XMLChStr (const XMLCh * const toTranscode)
        \brief Constructor.

        \param toTranscode XMLCh* data to convert
    */
    XMLChStr (const XMLCh * const toTranscode);


    /*! \fn virtual ~XMLChStr ()
        \brief Destructor.

        Release memory.
    */
    virtual ~XMLChStr ();


    /*! \fn const XMLCh * UnicodeForm () const
        \brief Get converted unicode form.

        \return converted unicode data.
    */
    const XMLCh * UnicodeForm () const;


    /*! \fn const XMLCh * CharForm () const
        \brief Get converted char form.

        \return converted char data.
    */
    const char * CharForm () const;

private:
    XMLCh *m_unicodeForm;
    char *m_charForm;

private:
    // No copy and assignment constructor:
    XMLChStr (const XMLChStr &rhs);
    XMLChStr & operator= (const XMLChStr &rhs);
};

} // End namespace sw

#endif // SWXMLCHSTR_H

