///////////////////////////////////////////////////////////////////////////////
// Name:        swWxGuiTesting/swCRXRCReader.cpp
// Author:      Reinhold Freder
// Created:     2004
// Copyright:   (c) 2005 Reinhold Freder
// Licence:     wxWindows licence
///////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
    #pragma implementation "swCRXRCReader.h"
#endif

#include "swCRXRCReader.h"

#include "xercesc/util/PlatformUtils.hpp"
#include "xercesc/dom/DOM.hpp"
#include "xercesc/parsers/SAXParser.hpp"

#include "wx/filename.h"

#include "swWxLogicErrorException.h"
#include "swXMLChStr.h"

#include "swCRXRCSAXHandler.h"

#include <iostream>

namespace swTst {


CRXRCReader::CRXRCReader (const wxString &filename)
{
    wxASSERT (!filename.IsEmpty ());
    wxFileName filenameObj (filename);
    bool isOk = filenameObj.MakeAbsolute ();
    wxASSERT (isOk);
    m_filename = filenameObj.GetFullPath ();
}


CRXRCReader::~CRXRCReader ()
{
    // Nothing to do
}


void writexml (xercesc::DOMNode *node)
{
    xercesc::DOMElement *elem = dynamic_cast< xercesc::DOMElement * >(node);
    if (elem == NULL)
        return;

    xercesc::DOMNodeList *rootNodeList = elem->getChildNodes ();
    wxASSERT (rootNodeList != NULL);

    for (int i = 0; i < rootNodeList->getLength (); i++) {
        xercesc::DOMNode *node2 = rootNodeList->item (i);
        wxString nodeName = sw::XMLChStr (node2->getNodeName ()).CharForm ();
        std::cout << nodeName.c_str () << " " << std::endl;
        wxString nodeValue = sw::XMLChStr (node2->getNodeValue ()).CharForm ();
        std::cout << nodeValue.c_str () << " " << std::endl;

        writexml (node2);
    }
}


CRXRCResource * CRXRCReader::GetResource ()
{
    xercesc::SAXParser *parser = NULL;
    xercesc::ErrorHandler *errHandler = NULL;
    CRXRCSAXHandler *docHandler = NULL;
    CRXRCResource *resource = NULL;
    bool isFailed = false;
    wxString failMsg;

    try {

        xercesc::XMLPlatformUtils::Initialize ();

        parser = new xercesc::SAXParser ();

        docHandler = new CRXRCSAXHandler ();
        parser->setDocumentHandler (docHandler);
        xercesc::ErrorHandler *errHandler = (xercesc::ErrorHandler *) docHandler;
        parser->setErrorHandler (errHandler);

        resource = new CRXRCResource ();
        docHandler->SetContextNode (resource);

        if (::wxFileExists (m_filename)) {

            parser->parse (m_filename);

        } else if (::wxDirExists (m_filename)) {

            // Should we restrict looking in sub directories?
            wxString file = ::wxFindFirstFile (m_filename + "/*.xrc" /*, wxFILE*/);
            while (!file.IsEmpty ()) {

                parser->parse (file);
                file = ::wxFindNextFile ();
            }

        } else {

            isFailed = true;
            failMsg = wxString::Format ("%s: %s",
                    _("Given filename is neither a valid file, nor a path"),
                    m_filename.c_str ());
        }

    } catch (const xercesc::XMLException &toCatch) {

        isFailed = true;
        failMsg = wxString::Format ("%s: %s", _("XML error"),
                sw::XMLChStr (toCatch.getMessage ()).CharForm ());

    } catch (const xercesc::DOMException &toCatch) {

        isFailed = true;
        failMsg = wxString::Format ("%s: %s", _("XML error"),
                sw::XMLChStr (toCatch.msg).CharForm ());

    } catch (std::exception &e) {

        isFailed = true;
        failMsg = wxString::Format ("%s: %s", _("XML error"), e.what ());

    } catch (...) {

        isFailed = true;
        failMsg = wxString::Format ("%s", _("XML error"));
    }
    // Clean up:
    if (parser != NULL) {

        delete parser;
    }
    if (docHandler != NULL) {

        delete docHandler; // == errHandler, so do not delete it twice
    }

    xercesc::XMLPlatformUtils::Terminate();

    if (isFailed) {

        throw sw::WxLogicErrorException (failMsg);
    }

    return resource;
}

} // End namespace swTst

